unit NPCAI;

interface

uses
  Classes;

type
  TNPCAIEvent = procedure(var aX: Extended; var aY: Extended;
                          var aSpeed: Extended) of object;

  TNPCAI = class
  protected
    FX, FY: Extended; // Trenutna X i Y koordinata
    FSpeed: Extended; // Trenutna brzina (sluzi za racunanje sledece pozicije)
    function GetIntSpeed: Integer;
    function GetIntX: Integer;
    function GetIntY: Integer;
  public
    constructor Create(aX, aY, aSpeed: Extended); // Pocetna X i Y koordinata i pocetna brzina
    procedure Tick; virtual; abstract; // Kada zelis da se X i Y pomere na sledecu tacku pozoves Tick
    property X: Extended read FX; // Vraca trenutnu X koordinatu
    property Y: Extended read FY; // Vraca trenutnu X koordinatu
    property Speed: Extended read FSpeed; // Vraca trenutnu brzinu
    property IntX: Integer read GetIntX; // Vraca trenutnu X koordinatu (integer)
    property IntY: Integer read GetIntY; // Vraca trenutnu Y koordinatu (integer)
    property IntSpeed: Integer read GetIntSpeed; // Vraca trenutnu brzinu (integer)
  end;

  TCustomNPCAI = class(TNPCAI)
  protected
    FOnTick: TNPCAIEvent; // Kad god se pozove Tick OnTick event treba da promeni poziciju uzimajuci u obzir trenutnu poziciju i brzinu
  public
    procedure Tick; override; // Tick...
    property OnTick: TNPCAIEvent read FOnTick write FOnTick; // Postavljanje OnTick eventa
  end;

  TCircleNPCAI = class(TNPCAI)
  protected
    FCenterX, FCenterY, FRadius, FAngle: Extended; // X i Y pozicija centra oko kojeg se objekat okrece, poluprecnik, i trenutni ugao
    procedure CalcPos; // Izracunava trenutnu poziciju na osnovu centra, poluprecnika i ugla
    function GetIntAngle: Integer;
    function GetIntCenterX: Integer;
    function GetIntCenterY: Integer;
    function GetIntRadius: Integer;
  public
    constructor Create(aCenterX, aCenterY, aRadius,
                       aAngle, aSpeed: Extended); // Postavljanje pocetnih vrednosti
    procedure Tick; override; // Tick...
    property CenterX: Extended read FCenterX; // Citanje vrednosti
    property CenterY: Extended read FCenterY;
    property Radius: Extended read FRadius;
    property Angle: Extended read FAngle;
    property IntCenterX: Integer read GetIntCenterX; // Citanje vrednosti (integer)
    property IntCenterY: Integer read GetIntCenterY;
    property IntRadius: Integer read GetIntRadius;
    property IntAngle: Integer read GetIntAngle;
  end;

  THorisontalNPCAI = class(TNPCAI)
  private
    function GetIntMaxX: Integer;
    function GetIntMinX: Integer;
  protected
    FMinX, FMaxX: Extended; // Minimalna i maximalna vrednost za X koordinatu izmedju koje ce objekat da se krece
    procedure CheckPos; // Proverava da li je X koordinata u dozvoljenim granicama i menja smer ako je potrebno
  public
    constructor Create(aX, aY, aMinX, aMaxX, aSpeed: Extended); // Pocetne vrednosti (ako je aSpeed pozitivna vrednost objekat krece desno, a ako je negativna onda levo)
    procedure Tick; override; // Tick...
    property MinX: Extended read FMinX; // Citanje vrednosti
    property MaxX: Extended read FMaxX;
    property IntMinX: Integer read GetIntMinX; // Citanje vrednosti (integer)
    property IntMaxX: Integer read GetIntMaxX;
  end;

implementation

function DegToRad(X: Extended): Extended;
begin
  Result := X * Pi / 180;
end;

{ TNPCAI }

constructor TNPCAI.Create(aX, aY, aSpeed: Extended);
begin
  FX := aX;
  FY := aY;
  FSpeed := aSpeed;
end;

function TNPCAI.GetIntSpeed: Integer;
begin
  Result := Trunc(FSpeed);
end;

function TNPCAI.GetIntX: Integer;
begin
  Result := Trunc(FX);
end;

function TNPCAI.GetIntY: Integer;
begin
  Result := Trunc(FY);
end;

{ TCustomNPCAI }

procedure TCustomNPCAI.Tick;
begin
  if Assigned(FOnTick) then FOnTick(FX, FY, FSpeed);
end;

{ TCircleNPCAI }

procedure TCircleNPCAI.CalcPos;
begin
  FX := Cos(DegToRad(FAngle)) * FRadius + FCenterX;
  FY := Sin(DegToRad(FAngle)) * FRadius + FCenterY;
end;

constructor TCircleNPCAI.Create(aCenterX, aCenterY, aRadius, aAngle,
  aSpeed: Extended);
begin
  FCenterX := aCenterX;
  FCenterY := aCenterY;
  FRadius := aRadius;
  FAngle := aAngle;
  While FAngle >= 360 do FAngle := FAngle - 360;
  FSpeed := aSpeed;
  CalcPos;
end;

function TCircleNPCAI.GetIntAngle: Integer;
begin
  Result := Trunc(FAngle);
end;

function TCircleNPCAI.GetIntCenterX: Integer;
begin
  Result := Trunc(FCenterX);
end;

function TCircleNPCAI.GetIntCenterY: Integer;
begin
  Result := Trunc(FCenterY);
end;

function TCircleNPCAI.GetIntRadius: Integer;
begin
  Result := Trunc(FRadius);
end;

procedure TCircleNPCAI.Tick;
begin
  FAngle := FAngle + FSpeed;
  While FAngle >= 360 do FAngle := FAngle - 360;
  CalcPos;
end;

{ THorisontalNPCAI }

procedure THorisontalNPCAI.CheckPos;
begin
  if FX <= FMinX then
  begin
    FX := FMinX;
    FSpeed := Abs(FSpeed);
  end;
  if FX >= FMaxX then
  begin
    FX := FMaxX;
    FSpeed := -Abs(FSpeed);
  end;
end;

constructor THorisontalNPCAI.Create(aX, aY, aMinX, aMaxX,
  aSpeed: Extended);
begin
  FX := aX;
  FY := aY;
  FMaxX := aMaxX;
  FMinX := aMinX;
  FSpeed := aSpeed;
  CheckPos;
end;

function THorisontalNPCAI.GetIntMaxX: Integer;
begin
  Result := Trunc(FMaxX);
end;

function THorisontalNPCAI.GetIntMinX: Integer;
begin
  Result := Trunc(FMinX);
end;

procedure THorisontalNPCAI.Tick;
begin
  FX := FX + FSpeed;
  CheckPos;
end;

end.
